use ewebsock::WsEvent;
use macroquad::prelude::{error, info};
use serde::{Serialize, de::DeserializeOwned};

pub struct Context {
    pub sender: ewebsock::WsSender,
    pub reciever: ewebsock::WsReceiver,
    pub token: String,
    pub username: String,
}

impl Context {
    pub fn new(url: &str) -> Context {
        let (sender, reciever) = ewebsock::connect(url, ewebsock::Options::default()).unwrap();

        Self {
            sender,
            reciever,
            token: String::new(),
            username: String::new(),
        }
    }

    pub fn send_msg<T: Serialize>(&mut self, msg: &T) {
        let msg = serde_json::to_string(msg).unwrap();
        self.sender.send(ewebsock::WsMessage::Text(msg));
    }

    pub fn recv_msg<T: DeserializeOwned>(&mut self) -> Option<T> {
        if let Some(msg) = self.reciever.try_recv() {
            match msg {
                WsEvent::Opened => {
                    info!("Web socket opened!");
                    None
                }
                WsEvent::Message(ws_message) => match ws_message {
                    ewebsock::WsMessage::Text(msg) => {
                        let game_resp: T = serde_json::from_str(&msg).unwrap();

                        Some(game_resp)
                    }
                    _ => None,
                },
                WsEvent::Error(err) => {
                    error!("Something hecky going on: {}", err);
                    None
                }
                WsEvent::Closed => {
                    panic!("Socket closed, something up...");
                }
            }
        } else {
            None
        }
    }
}
